// ignore_for_file: library_private_types_in_public_api, no_logic_in_create_state, use_key_in_widget_constructors, prefer_const_constructors_in_immutables

import 'package:progamification/src/widgets/achievements.dart';
import 'package:progamification/src/widgets/leaderboard_table.dart';
import 'package:progamification/src/controllers/leader_board.dart';
import 'package:get/get.dart';
import 'package:flutter/material.dart';
import 'package:progamification/src/services/leaderboard.service.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:aligned_dialog/aligned_dialog.dart';

class LeaderBoard extends StatefulWidget {
  final String userId;
  final String applicationId;
  LeaderBoard(this.userId, this.applicationId);
  @override
  _LeaderBoardState createState() => _LeaderBoardState(userId, applicationId);
}

class _LeaderBoardState extends State<LeaderBoard> {
  late String userId;
  late String applicationId;
  bool isGamificationVisible = true;
  _LeaderBoardState(this.userId, this.applicationId);

  LeaderBoardController leaderBoardController =
      Get.find<LeaderBoardController>();

  LeaderBoardService leaderBoardService = Get.find();
  @override
  void initState() {
    super.initState();
    getData(userId, applicationId);
  }

  Future<void> getData(String userId, String applicationId) async {
    try {
      await leaderBoardService
          .getLeaderBoard(userId, applicationId)
          .then(leaderBoardController.setData);
    } catch (e) {
      setState(() {
        isGamificationVisible = false;
      });
    }
  }

  TextStyle headerStyle = TextStyle(
    letterSpacing: 1.0,
    fontWeight: FontWeight.w600,
    fontSize: 19.0,
    fontFamily: GoogleFonts.plusJakartaSans().fontFamily,
  );
  @override
  Widget build(BuildContext context) {
    return isGamificationVisible
        ? Container(
            padding: const EdgeInsets.only(left: 0, right: 0),
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(10),
            ),
            child: Row(
              children: [
                InkWell(
                  onTap: (() async {
                    await leaderBoardController.getData(userId, applicationId);

                    if (kIsWeb) {
                      // ignore: use_build_context_synchronously
                      showAlignedDialog(
                          context: context,
                          followerAnchor: Alignment.topRight,
                          targetAnchor: Alignment.bottomRight,
                          barrierColor: Colors.transparent,
                          builder: (BuildContext context) => Container(
                              margin: const EdgeInsets.only(top: 10, right: 10),
                              decoration: BoxDecoration(
                                boxShadow: [
                                  BoxShadow(
                                    color: Colors.grey.withOpacity(0.5),
                                    spreadRadius: 5,
                                    blurRadius: 7,
                                    offset: const Offset(3, 3),
                                  ),
                                ],
                              ),
                              child: ClipRRect(
                                  borderRadius: const BorderRadius.only(
                                    topLeft: Radius.circular(10.0),
                                    topRight: Radius.circular(10.0),
                                    bottomLeft: Radius.circular(10.0),
                                    bottomRight: Radius.circular(10.0),
                                  ),
                                  child: tabContent(context))));
                    } else {
                      showLeaderBoard();
                    }
                  }),
                  child: Container(
                    margin: const EdgeInsets.all(5),
                    child: const Icon(
                      Icons.sports_score_outlined,
                      color: Colors.black,
                    ),
                  ),
                ),
              ],
            ),
          )
        : Container();
  }

  showLeaderBoard() {
    showModalBottomSheet(
        backgroundColor: Colors.white,
        context: context,
        isScrollControlled: true,
        builder: (BuildContext context) => tabContent(context));
  }

  SizedBox tabContent(context) => SizedBox(
        height: kIsWeb ? Get.height - 75 : Get.height,
        width: kIsWeb ? 450 : double.infinity,
        child: DefaultTabController(
          length: 2,
          child: Scaffold(
            appBar: AppBar(
              backgroundColor: Colors.white,
              automaticallyImplyLeading: false,
              toolbarHeight: kTextTabBarHeight, //kToolbarHeight,
              flexibleSpace: kIsWeb
                  ? null
                  : Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: [
                        PreferredSize(
                            preferredSize: const Size.fromHeight(80),
                            child: Container(
                                margin:
                                    const EdgeInsets.only(top: 45.0, left: 20),
                                child: FloatingActionButton(
                                  onPressed: (() => Navigator.pop(context)),
                                  elevation: 0,
                                  backgroundColor: Colors.grey[100],
                                  child: const Icon(
                                    Icons.close_sharp,
                                    color: Colors.black,
                                  ),
                                )))
                      ],
                    ),
              bottom: PreferredSize(
                  preferredSize: kIsWeb
                      ? const Size.fromHeight(2.5)
                      : const Size.fromHeight(100),
                  child: TabBar(
                    labelPadding: const EdgeInsets.symmetric(vertical: 0),
                    indicatorSize: kIsWeb
                        ? TabBarIndicatorSize.tab
                        : TabBarIndicatorSize.label,
                    labelColor: Colors.black87,
                    unselectedLabelColor: Colors.grey,
                    indicator: const UnderlineTabIndicator(
                      borderSide: BorderSide(width: 2, color: Colors.black87),
                      borderRadius: BorderRadius.only(
                        topLeft: Radius.circular(2),
                        topRight: Radius.circular(2),
                        bottomLeft: Radius.circular(2),
                        bottomRight: Radius.circular(2),
                      ),
                    ),
                    tabs: [
                      Tab(
                          child: Padding(
                        padding: const EdgeInsets.only(left: 10.0, right: 10.0),
                        child: Text(
                          "Achievements",
                          style: headerStyle,
                        ),
                      )),
                      Tab(
                        child: Padding(
                            padding:
                                const EdgeInsets.only(left: 10.0, right: 10.0),
                            child: Text(
                              "Leaderboard",
                              style: headerStyle,
                            )),
                      ),
                    ],
                  )),
            ),
            body: const TabBarView(
                children: [Achievements(), LeaderBoardTable()]),
          ),
        ),
      );
}
