import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:progamification/src/widgets/avatar_placeholder.dart';
import 'package:progamification/src/widgets/bolt.dart';
import 'package:progamification/src/widgets/rank.dart';
import 'package:progamification/src/constants/color_contants.dart';
import 'package:progamification/src/controllers/leader_board.dart';
import 'package:progamification/src/models/leader_board.dart';
import 'package:progamification/src/services/sas_token.service.dart';
import 'package:progamification/src/services/storage.service.dart';
import 'package:scrollable_positioned_list/scrollable_positioned_list.dart';
import 'package:google_fonts/google_fonts.dart';

class LeaderBoardTable extends StatefulWidget {
  const LeaderBoardTable({super.key});

  @override
  State<LeaderBoardTable> createState() => _LeaderBoardTableState();
}

class _LeaderBoardTableState extends State<LeaderBoardTable> {
  final ItemScrollController itemScrollController = ItemScrollController();
  final ItemPositionsListener itemPositionsListener =
      ItemPositionsListener.create();

  LeaderBoardController leaderBoardController =
      Get.find<LeaderBoardController>();
  GamificationStorageService storageService =
      Get.find<GamificationStorageService>();
  LeaderBoardModel currentUser = LeaderBoardModel();
  SasTokenService sasTokenService = Get.find<SasTokenService>();
  String sasToken = "";
  String userId = '';

  @override
  initState() {
    super.initState();
    setCurrentUserLeaderBoardData();
    sasTokenService.getSasToken().then((value) {
      setState(() {
        sasToken = value.toString();
      });
    });
    storageService.getString('userId').then((value) => {
          setState(() {
            userId = value.toString();
          })
        });
  }

  TextStyle textStyle(fontSize, {Color color = Colors.black}) => TextStyle(
      color: color,
      fontWeight: FontWeight.w600,
      fontSize: fontSize,
      fontFamily: GoogleFonts.plusJakartaSans().fontFamily);
  @override
  Widget build(BuildContext context) {
    return SizedBox(
        height: MediaQuery.of(context).size.height,
        child: Scaffold(
            body: ScrollablePositionedList.builder(
              itemScrollController: itemScrollController,
              itemPositionsListener: itemPositionsListener,
              shrinkWrap: true,
              itemCount: leaderBoardController.leaderBoardData.length,
              itemBuilder: (BuildContext context, int index) {
                if (index == 0) {
                  return const SizedBox(
                    height: 220.0,
                    width: double.infinity,
                    child: Rank(),
                  );
                }
                if (index >= 3) {
                  Color rankColor = leaderBoardController
                              .leaderBoardData[index].userId
                              .toString() ==
                          userId
                      ? Colors.white
                      : Colors.black;
                  return ListTile(
                    shape: RoundedRectangleBorder(
                      side: BorderSide(
                          color: leaderBoardController
                                      .leaderBoardData[index].userId
                                      .toString() ==
                                  userId
                              ? ColorConstants.selectedTileColor
                              : const Color.fromARGB(255, 238, 234, 234),
                          width: 1),
                    ),
                    tileColor: Colors.white,
                    selectedTileColor: ColorConstants.selectedTileBgColor,
                    selectedColor: Colors.black,
                    contentPadding: const EdgeInsets.only(
                        left: 30, right: 20, top: 8.0, bottom: 8.0),
                    selected: leaderBoardController
                            .leaderBoardData[index].userId
                            .toString() ==
                        userId,
                    leading: Container(
                      decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        color: leaderBoardController
                                    .leaderBoardData[index].userId
                                    .toString() ==
                                userId
                            ? ColorConstants.selectedTileColor
                            : const Color(0xffEBEBEB),
                      ),
                      child: Padding(
                          padding: const EdgeInsets.all(7.75),
                          child: Text(
                            leaderBoardController.leaderBoardData[index].rank
                                .toString(),
                            textAlign: TextAlign.start,
                            style: textStyle(16.0, color: rankColor),
                          )),
                    ),
                    title: Row(children: [
                      Container(
                          margin: const EdgeInsets.only(right: 15.0),
                          width: 48.0,
                          height: 48.0,
                          decoration: const BoxDecoration(
                            shape: BoxShape.circle,
                          ),
                          child: ClipOval(
                              child: CachedNetworkImage(
                            key: UniqueKey(),
                            imageUrl: leaderBoardController
                                    .leaderBoardData[index].profileImageUrl
                                    .toString() +
                                sasToken,
                            width: 50.0,
                            height: 50.0,
                            fit: BoxFit.fill,
                            placeholder: (context, url) =>
                                const AvatarPlaceHolder(),
                            errorWidget: (context, url, error) =>
                                const AvatarPlaceHolder(),
                          ))),
                      Expanded(
                          child: Text(
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                        textAlign: TextAlign.left,
                        leaderBoardController.leaderBoardData[index].userName
                            .toString(),
                        style: textStyle(16.0),
                      )),
                    ]),
                    trailing: Row(mainAxisSize: MainAxisSize.min, children: [
                      const Bolt(
                        height: 55,
                        width: 55,
                      ),
                      Text(
                        '${leaderBoardController.leaderBoardData[index].score}',
                        style: textStyle(15.0),
                      )
                    ]),
                  );
                } else {
                  return Container();
                }
              },
            ),
            floatingActionButton: FloatingActionButton.extended(
              extendedPadding: const EdgeInsets.symmetric(horizontal: 0),
              backgroundColor: Colors.black,
              onPressed: () {
                int selectedIndex =
                    leaderBoardController.getCurrentUserIndex(userId)!;
                scrollToIndex(selectedIndex);
              },
              label: Row(children: [
                Container(
                    margin: const EdgeInsets.symmetric(horizontal: 10.0),
                    width: 33.0,
                    height: 33.0,
                    decoration: const BoxDecoration(
                      shape: BoxShape.circle,
                    ),
                    child: ClipOval(
                        child: CachedNetworkImage(
                      key: UniqueKey(),
                      imageUrl: currentUser.profileImageUrl + sasToken,
                      fit: BoxFit.fill,
                      placeholder: (context, url) => Image.asset(
                        "assets/icons/avatar.png",
                        package: "progamification",
                      ),
                      errorWidget: (context, url, error) => Image.asset(
                        "assets/icons/avatar.png",
                        package: "progamification",
                      ),
                    ))),
                Text("You're here",
                    style: TextStyle(
                        color: Colors.white,
                        fontFamily: GoogleFonts.plusJakartaSans().fontFamily)),
                const Icon(Icons.arrow_drop_down)
              ]),
            )));
  }

  Future<void> setCurrentUserLeaderBoardData() async {
    String userId = await storageService.getString('userId');
    LeaderBoardModel leaderboard =
        leaderBoardController.getLeaderBoardData(userId);
    setState(() {
      currentUser = leaderboard;
    });
  }

  void scrollToIndex(int index) {
    itemScrollController.scrollTo(
        alignment: 0.5,
        index: index,
        duration: const Duration(seconds: 2),
        curve: Curves.easeInOutCubic);
  }
}
